@@ -47,9 +47,9 @@ static Pattern symbolPat = Pattern.compile("[:]?([\\D&&[^/]].*/)?([\\D&&[^/]][^/
 //static Pattern intPat = Pattern.compile("[-+]?[0-9]+\\.?");
 static Pattern intPat =
 		Pattern.compile(
-				"([-+]?)(?:(0)|([1-9][0-9]*)|0[xX]([0-9A-Fa-f]+)|0([0-7]+)|([1-9][0-9]?)[rR]([0-9A-Za-z]+)|0[0-9]+)\\.?");
+				"([-+]?)(?:(0)|([1-9][0-9]*)|0[xX]([0-9A-Fa-f]+)|0([0-7]+)|([1-9][0-9]?)[rR]([0-9A-Za-z]+)|0[0-9]+)");
 static Pattern ratioPat = Pattern.compile("([-+]?[0-9]+)/([0-9]+)");
-static Pattern floatPat = Pattern.compile("[-+]?[0-9]+(\\.[0-9]+)?([eE][-+]?[0-9]+)?[M]?");
+static Pattern floatPat = Pattern.compile("([-+]?[0-9]+(\\.[0-9]*)?([eE][-+]?[0-9]+)?)(M)?");
 static final Symbol SLASH = Symbol.create("/");
 static final Symbol CLOJURE_SLASH = Symbol.create("clojure.core","/");
 //static Pattern accessorPat = Pattern.compile("\\.[a-zA-Z_]\\w*");
@@ -68,7 +68,7 @@ static Var ARG_ENV = Var.create(null);
 	macros[';'] = new CommentReader();
 	macros['\''] = new WrappingReader(QUOTE);
 	macros['@'] = new WrappingReader(DEREF);//new DerefReader();
-	macros['^'] = new WrappingReader(META);
+	macros['^'] = new DeprecatedWrappingReader(META, "^");
 	macros['`'] = new SyntaxQuoteReader();
 	macros['~'] = new UnquoteReader();
 	macros['('] = new ListReader();
@@ -344,8 +344,8 @@ private static Object matchNumber(String s){
 	m = floatPat.matcher(s);
 	if(m.matches())
 		{
-		if(s.charAt(s.length() - 1) == 'M')
-			return new BigDecimal(s.substring(0, s.length() - 1));
+		if(m.group(4) != null)
+			return new BigDecimal(m.group(1));
 		return Double.parseDouble(s);
 		}
 	m = ratioPat.matcher(s);
@@ -491,6 +491,26 @@ public static class WrappingReader extends AFn{
 
 }
 
+public static class DeprecatedWrappingReader extends AFn{
+	final Symbol sym;
+        final String macro;
+
+	public DeprecatedWrappingReader(Symbol sym, String macro){
+		this.sym = sym;
+                this.macro = macro;
+	}
+
+	public Object invoke(Object reader, Object quote) throws Exception{
+                System.out.println("WARNING: reader macro " + macro +
+                                   " is deprecated; use " + sym.getName() +
+                                   " instead");
+		PushbackReader r = (PushbackReader) reader;
+		Object o = read(r, true, null, true);
+		return RT.list(sym, o);
+	}
+
+}
+
 public static class VarReader extends AFn{
 	public Object invoke(Object reader, Object quote) throws Exception{
 		PushbackReader r = (PushbackReader) reader;
@@ -607,6 +627,10 @@ static Symbol registerArg(int n){
 static class ArgReader extends AFn{
 	public Object invoke(Object reader, Object pct) throws Exception{
 		PushbackReader r = (PushbackReader) reader;
+		if(ARG_ENV.deref() == null)
+			{
+			return interpretToken(readToken(r, '%'));
+			}
 		int ch = r.read();
 		unread(r, ch);
 		//% alone is first arg
@@ -700,7 +724,20 @@ public static class SyntaxQuoteReader extends AFn{
 				// Simply quote method names.
  				}
             else
-				sym = Compiler.resolveSymbol(sym);
+				{
+					Object maybeClass = null;
+					if(sym.ns != null)
+						maybeClass = Compiler.currentNS().getMapping(
+								Symbol.intern(null, sym.ns));
+					if(maybeClass instanceof Class)
+						{
+						// Classname/foo -> package.qualified.Classname/foo
+						sym = Symbol.intern(
+								((Class)maybeClass).getName(), sym.name);
+						}
+					else
+						sym = Compiler.resolveSymbol(sym);
+				}
 			ret = RT.list(Compiler.QUOTE, sym);
 			}
 		else if(isUnquote(form))
