@@ -179,12 +179,13 @@ final static public Var ERR =
 final static Keyword TAG_KEY = Keyword.intern(null, "tag");
 final static public Var AGENT = Var.intern(CLOJURE_NS, Symbol.create("*agent*"), null);
 final static public Var READEVAL = Var.intern(CLOJURE_NS, Symbol.create("*read-eval*"), T);
+final static public Var ASSERT = Var.intern(CLOJURE_NS, Symbol.create("*assert*"), T);
 final static public Var MACRO_META = Var.intern(CLOJURE_NS, Symbol.create("*macro-meta*"), null);
 final static public Var MATH_CONTEXT = Var.intern(CLOJURE_NS, Symbol.create("*math-context*"), null);
 static Keyword LINE_KEY = Keyword.intern(null, "line");
 static Keyword FILE_KEY = Keyword.intern(null, "file");
 final static public Var USE_CONTEXT_CLASSLOADER =
-		Var.intern(CLOJURE_NS, Symbol.create("*use-context-classloader*"), null);
+		Var.intern(CLOJURE_NS, Symbol.create("*use-context-classloader*"), T);
 //final static public Var CURRENT_MODULE = Var.intern(Symbol.create("clojure.core", "current-module"),
 //                                                    Module.findOrCreateModule("clojure/user"));
 
@@ -236,11 +237,14 @@ static public final Comparator DEFAULT_COMPARATOR = new Comparator(){
 };
 
 static AtomicInteger id = new AtomicInteger(1);
-private static DynamicClassLoader ROOT_CLASSLOADER = null;
 
 static public void addURL(Object url) throws Exception{
 	URL u = (url instanceof String) ? (new URL((String) url)) : (URL) url;
-	getRootClassLoader().addURL(u);
+	ClassLoader ccl = Thread.currentThread().getContextClassLoader();
+	if(ccl instanceof DynamicClassLoader)
+		((DynamicClassLoader)ccl).addURL(u);
+	else
+		throw new IllegalAccessError("Context classloader is not a DynamicClassLoader");
 }
 
 final static public Object EOS = new Object();
@@ -472,7 +476,7 @@ static ISeq seqFrom(Object coll){
 	else {
 		Class c = coll.getClass();
 		Class sc = c.getSuperclass();
-		throw new IllegalArgumentException("Don't know how to create ISeq from: " + c.getSimpleName());
+		throw new IllegalArgumentException("Don't know how to create ISeq from: " + c.getName());
 	}
 }
 
@@ -490,11 +494,17 @@ static public IPersistentMap meta(Object x){
 	return null;
 }
 
+public static int count(Counted o){
+	if(o != null)
+		return o.count();
+	return 0;
+}
+
 public static int count(Object o){
+	if(o instanceof Counted)
+		return ((Counted) o).count();
 	if(o == null)
 		return 0;
-	else if(o instanceof Counted)
-		return ((Counted) o).count();
 	else if(o instanceof IPersistentCollection) {
 		ISeq s = seq(o);
 		o = null;
@@ -605,8 +615,8 @@ static public Object pop(Object x){
 static public Object get(Object coll, Object key){
 	if(coll == null)
 		return null;
-	else if(coll instanceof Associative)
-		return ((Associative) coll).valAt(key);
+	else if(coll instanceof ILookup)
+		return ((ILookup) coll).valAt(key);
 	else if(coll instanceof Map) {
 		Map m = (Map) coll;
 		return m.get(key);
@@ -628,8 +638,8 @@ static public Object get(Object coll, Object key){
 static public Object get(Object coll, Object key, Object notFound){
 	if(coll == null)
 		return notFound;
-	else if(coll instanceof Associative)
-		return ((Associative) coll).valAt(key, notFound);
+	else if(coll instanceof ILookup)
+		return ((ILookup) coll).valAt(key, notFound);
 	else if(coll instanceof Map) {
 		Map m = (Map) coll;
 		if(m.containsKey(key))
@@ -709,10 +719,10 @@ static public Object dissoc(Object coll, Object key) throws Exception{
 }
 
 static public Object nth(Object coll, int n){
+	if(coll instanceof Indexed)
+		return ((Indexed) coll).nth(n);
 	if(coll == null)
 		return null;
-	else if(coll instanceof IPersistentVector)
-		return ((IPersistentVector) coll).nth(n);
 	else if(coll instanceof String)
 		return Character.valueOf(((String) coll).charAt(n));
 	else if(coll.getClass().isArray())
@@ -746,16 +756,17 @@ static public Object nth(Object coll, int n){
 }
 
 static public Object nth(Object coll, int n, Object notFound){
-	if(coll == null)
-		return notFound;
-	else if(n < 0)
-		return notFound;
-	else if(coll instanceof IPersistentVector) {
-		IPersistentVector v = (IPersistentVector) coll;
-		if(n < v.count())
+	if(coll instanceof Indexed) {
+		Indexed v = (Indexed) coll;
+		if(n >= 0 && n < v.count())
 			return v.nth(n);
 		return notFound;
 	}
+	else if(coll == null)
+		return notFound;
+	else if(n < 0)
+		return notFound;
+
 	else if(coll instanceof String) {
 		String s = (String) coll;
 		if(n < s.length())
@@ -1468,17 +1479,23 @@ static public Object[] setValues(Object... vals){
 static public ClassLoader makeClassLoader(){
 	return (ClassLoader) AccessController.doPrivileged(new PrivilegedAction(){
 		public Object run(){
-			getRootClassLoader();
+            try{
+            Var.pushThreadBindings(RT.map(USE_CONTEXT_CLASSLOADER, RT.T));
+//			getRootClassLoader();
 			return new DynamicClassLoader(baseLoader());
+            }
+                finally{
+            Var.popThreadBindings();
+            }
 		}
 	});
 }
 
 static public ClassLoader baseLoader(){
-	if(booleanCast(USE_CONTEXT_CLASSLOADER.deref()))
+	if(Compiler.LOADER.isBound())
+		return (ClassLoader) Compiler.LOADER.deref();
+	else if(booleanCast(USE_CONTEXT_CLASSLOADER.deref()))
 		return Thread.currentThread().getContextClassLoader();
-	else if(ROOT_CLASSLOADER != null)
-		return ROOT_CLASSLOADER;
 	return Compiler.class.getClassLoader();
 }
 
@@ -1665,12 +1682,4 @@ static public int alength(Object xs){
 	return Array.getLength(xs);
 }
 
-
-
-
-synchronized public static DynamicClassLoader getRootClassLoader(){
-	if(ROOT_CLASSLOADER == null)
-		ROOT_CLASSLOADER = new DynamicClassLoader();
-	return ROOT_CLASSLOADER;
-}
 }
